package com.ldodds.twinkle.ui.action;

import java.io.ByteArrayOutputStream;
import java.io.PrintWriter;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JOptionPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.text.Element;

import org.jdesktop.swingworker.SwingWorker;

import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryException;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.query.ResultSetFactory;
import com.hp.hpl.jena.query.ResultSetFormatter;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.shared.PrefixMapping;
import com.hp.hpl.jena.sparql.resultset.ResultSetException;
import com.hp.hpl.jena.util.FileUtils;
import com.ldodds.twinkle.ui.components.JTableResultsSetFormatter;
import com.ldodds.twinkle.ui.components.QueryInterface;

public class QueryWorker extends SwingWorker<Object,String>
{
	private QueryInterface queryInterface;
	private Query query;
	private QueryExecution queryExecution;
	
	public QueryWorker(QueryInterface queryInterface) {
		this.queryInterface = queryInterface;
		queryInterface.setSwingWorker(this);   		
	}

	@Override
	protected Object doInBackground() throws Exception {
		return doQuery();
	}
	
	// select line and column number in a textarea
	private void selectLineCol(JTextArea ta, int line, int col){
//		System.out.println("selectLineCol("+ta+","+line+","+col+")");
		Element root = ta.getDocument().getDefaultRootElement();
		int pos=root.getElement(line-1).getStartOffset()+col-1;
		ta.select(pos,pos+1);
	}
	
	private static Pattern p = Pattern.compile("line (\\d+), column (\\d+)");
	
    private Object doQuery() {    	
        query = null;
        try {
        	query = queryInterface.getQuery();
        } catch (QueryException qe1) {
        	String mess=qe1.getMessage();
        	Matcher m = p.matcher(mess);
        	if(m.find()){// message contains a line and column number...
        		int line=Integer.parseInt(m.group(1));
        		int column=Integer.parseInt(m.group(2));
        		selectLineCol(queryInterface.getQueryField(),line,column);
        	}
        	JOptionPane.showMessageDialog(null, mess, "Error in parsing query", JOptionPane.ERROR_MESSAGE);
        	//FIXME add catch for ResultSetException
//        	ErrorInfo errorInfo = new ErrorInfo("Query Error", qe1.getMessage(), "", 
//        			null, qe1, Level.SEVERE, null);
//        	JXErrorPane.showDialog(
//        			(java.awt.Component)ComponentRegistry.registry.getMainWindow(), errorInfo);        	
//        	qe1.printStackTrace();
        	return null;
        }
        
        queryExecution = null;
        try {
        	queryExecution = queryInterface.getQueryExecution(query);
        } catch (ResultSetException de) {
        	Throwable toReport = de.getCause() != null ? de.getCause() : de;
        	String msg = toReport.getMessage();
        	if ( msg != null && msg.indexOf("Interrupt detected") != -1 ) {
        		//then this exception thrown by XML Handler because we've
        		//cancelled, so ignore it
        		return null;
        	}
        	JOptionPane.showMessageDialog(null, msg, "Error in query execution", JOptionPane.ERROR_MESSAGE);
//        	ErrorInfo errorInfo = new ErrorInfo("Data Error", toReport.getMessage(), "", 
//        			null, toReport, Level.SEVERE, null);
//        	JXErrorPane.showDialog(
//        			(java.awt.Component)ComponentRegistry.registry.getMainWindow(), errorInfo);        	
//        	toReport.printStackTrace();
        	return null;
        	
        } catch (QueryException qe2) {
        	JOptionPane.showMessageDialog(null, qe2.getMessage(), "Error in query execution", JOptionPane.ERROR_MESSAGE);
//       	//FIXME try and differentiate RDF errors from query errors
//        	ErrorInfo errorInfo = new ErrorInfo("Query Execution Error", qe2.getMessage(), "", 
//        			null, qe2, Level.SEVERE, null);
//        	JXErrorPane.showDialog(
//        			(java.awt.Component)ComponentRegistry.registry.getMainWindow(), errorInfo);        	
//        	qe2.printStackTrace();
        	return null;
        }
        return execute(queryExecution);
    }
    
    private Object execute(QueryExecution queryExecution) 
    	throws QueryException
    {
        try
        {
        	if ( query.isSelectType() ) {
        		return queryExecution.execSelect();
        	}
        	else if ( query.isAskType() ) {
        		return Boolean.valueOf( queryExecution.execAsk() );
        	}
        	else if ( query.isConstructType() ) {
        		return queryExecution.execConstruct();
        	}
        	else {
        		return queryExecution.execDescribe();
        	}
        }
        catch (com.hp.hpl.jena.sparql.ARQInternalErrorException intEx)
        {
            //TEMPORARY... do we still need this?
            if (intEx.getCause() != null)
            {
                System.err.println("Cause:");
                intEx.getCause().printStackTrace(System.err);
                System.err.println();
            }
            intEx.printStackTrace(System.err);
            throw intEx;
        }
        catch (QueryException qEx)
        {
        	//FIXME error reporting
        	JOptionPane.showMessageDialog(null, qEx.getMessage(), "Error in query execution", JOptionPane.ERROR_MESSAGE);
            qEx.printStackTrace();
            throw qEx;
        }
        catch (Exception ex) {
        	//FIXME error reporting        	
        	JOptionPane.showMessageDialog(null, ex.getMessage(), "Error in query execution", JOptionPane.ERROR_MESSAGE);
            ex.printStackTrace();
            throw new QueryException("Unexpected error executing query", ex);
        }
    }
	
	protected void done() {		
		try {
			if ( this.isCancelled() ) {
				resetQuery();
				return;
			}
			
			Object o = get();
			
			if (o == null) {
				//we've already shown error dialog
				resetQuery();
				return;
			}
			boolean useShortForm=queryInterface.getOptions().getToolBar().useShortForm();
			PrefixMapping pm=queryInterface.getQuery().getPrefixMapping();
			// show first table output for SELECT and text for the others
    		JTabbedPane panel = queryInterface.getTabbedPanel();
    		int index = panel.indexOfTab(query.isSelectType()?"table":"text");
    		if (index != -1) {
    			panel.setSelectedIndex(index);        			
    		}
	        JTableResultsSetFormatter formatter = new JTableResultsSetFormatter();    	        
        	if ( query.isSelectType() ) {
        		ResultSet results = (ResultSet)o;
    	        results = ResultSetFactory.makeRewindable(results);
    	        formatter.display(queryInterface, results);    	        
                ByteArrayOutputStream bOut = new ByteArrayOutputStream();
                ResultSetFormatter.out(bOut, results, useShortForm?pm:null);        
                queryInterface.getTextResultsBox().setText(bOut.toString("UTF-8"));            	        
        	}
        	else if ( query.isAskType() ) {
        		Boolean result = (Boolean)o;
        		formatter.display(queryInterface,null);// clear table
                ByteArrayOutputStream bOut = new ByteArrayOutputStream();
                ResultSetFormatter.out(bOut, result);        
                queryInterface.getTextResultsBox().setText(bOut.toString("UTF-8"));        
        	}
        	else  {
        		Model results = (Model)o;
        		formatter.display(queryInterface,null);// clear table
                ByteArrayOutputStream bOut = new ByteArrayOutputStream();
                PrintWriter out = FileUtils.asPrintWriterUTF8(bOut);
                //TODO how do we choose output format?
                results.write(out,queryInterface.getOptions().getToolBar().getOutputFormat(),null);
                out.flush();
                out.close();        
                queryInterface.getTextResultsBox().setText(bOut.toString("UTF-8"));
        	}			
		} catch (InterruptedException ie) {
			//cancelled?
//			ie.printStackTrace();
		} catch (Exception e) {
        	JOptionPane.showMessageDialog(null, e.getMessage(), "Error in query execution", JOptionPane.ERROR_MESSAGE);
//        	ErrorInfo errorInfo = new ErrorInfo("Query Execution Error", e.getMessage(), "", 
//        			null, e, Level.SEVERE, null);
//        	JXErrorPane.showDialog(
//        			(java.awt.Component)ComponentRegistry.registry.getMainWindow(), errorInfo);        	
			e.printStackTrace();
		} finally {
			resetQuery();
		}
	}
	
	private void resetQuery(){
		queryInterface.setSwingWorker(null);
		if (queryExecution != null) {
			queryExecution.close();
		}
		queryInterface.setBusy(false);		
	}
    
}
