package com.ldodds.twinkle.ui.components;

import java.util.Map;
import java.util.HashMap;
import java.util.Vector;
import java.util.Iterator;

import javax.swing.JTable;
import javax.swing.table.*;

import com.hp.hpl.jena.query.*;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.shared.PrefixMapping;

/**
 * @author ccslrd
 * FIXME turn into JTable sub-class?
 */
public class JTableResultsSetFormatter
{
	PrefixMapping pm;
	boolean useShortForm;
	Map<String,String> blankNodeMap;
	
	public JTableResultsSetFormatter(){
		blankNodeMap=new HashMap<String,String>();
	}
	
	// this is probably already done in Jena but I could not find it
	private String getBlankId(String id){
		if(blankNodeMap.containsKey(id))
			return blankNodeMap.get(id);
		String newId="_:b"+blankNodeMap.size();
		blankNodeMap.put(id,newId);
		return newId;
	}
	
    public void display(QueryInterface queryFrame, ResultSet results)
    {
        pm=queryFrame.getQuery().getPrefixMapping();
        useShortForm = queryFrame.getOptions().getToolBar().useShortForm();
    	final JTable table = queryFrame.getResultsTable();        
        DefaultTableModel tableModel 
        	= results==null ? new DefaultTableModel() 
                            : new DefaultTableModel(getData(results), getColumns(results));
        table.setModel(tableModel);
        table.setAutoCreateRowSorter(true);
        table.repaint();
    }

    private Vector<Vector<String>> getData(ResultSet data)
    {
        Vector<Vector<String>> results = new Vector<Vector<String>>();
        for (; data.hasNext(); )
        {
            QuerySolution qs = data.nextSolution();
            results.add( getRowData(data, qs) );
        }
        return results;
    }
    
    @SuppressWarnings("unchecked")
	private Vector<String> getRowData(ResultSet results, QuerySolution qs)
    {
        Vector<String> row = new Vector<String>();
        for (Iterator<String> iter = results.getResultVars().iterator() ; iter.hasNext() ; )
        {
            String var = iter.next();
            row.add( getValueAsString(qs, var) );
        }
        return row;
    }
    
    private String getValueAsString(QuerySolution qs, String var)
    {
        RDFNode result = qs.get(var);
        if (result == null)
        {
            return "";
        }
        if(useShortForm){
	        if (result.isURIResource()){
	        	Resource res=(Resource)result;
	        	return pm.shortForm(res.getNameSpace())+res.getLocalName();
	        } else if (result.isLiteral()){
	        	String s=result.toString();
	        	int pos=s.indexOf("^^");
	        	if(pos>=0){
	        		return s.substring(0, pos+2)+pm.shortForm(s.substring(pos+2));
	        	}       	
	        } else if (result.isAnon()){
	        	return getBlankId(result.toString());
	        }
        }
        return result.toString();
    }
    
    @SuppressWarnings("unchecked")
	private Vector<String> getColumns(ResultSet results)
    {
        Vector<String> cols = new Vector<String>();
        cols.addAll(results.getResultVars());
        return cols;
    }
}
