package com.ldodds.twinkle.ui.components.menu;

import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.io.FileInputStream;
import java.io.FileNotFoundException;

import javax.swing.AbstractAction;
import javax.swing.ButtonGroup;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JScrollPane;
import javax.swing.KeyStroke;

import com.hp.hpl.jena.query.Syntax;
import com.hp.hpl.jena.util.FileManager;
import com.ldodds.twinkle.ui.action.CancelQueryAction;
import com.ldodds.twinkle.ui.action.RunQueryAction;
import com.ldodds.twinkle.ui.components.ComponentRegistry;

public class RunMenu extends TwinkleMenu
{
	public RunMenu() {
		super("Run", KeyEvent.VK_R);
		
		JMenuItem runMenuItem = 
			createMenuItem("Run", KeyEvent.VK_F5, 
					"/actions/media-playback-start.png", 
					KeyStroke.getKeyStroke(KeyEvent.VK_F5, 0), 
					new RunQueryAction());
		this.add(runMenuItem);

		JMenuItem cancelMenuItem = 
			createMenuItem("Cancel", KeyEvent.VK_F6, 
					"/actions/process-stop.png", 
					KeyStroke.getKeyStroke(KeyEvent.VK_F6, 0), 
					new CancelQueryAction());
		this.add(cancelMenuItem);
		
		this.addSeparator();
		
		JMenu syntaxMenuItem = new JMenu("Syntax");
		syntaxMenuItem.setMnemonic( KeyEvent.VK_Y );
		
		ButtonGroup group = new ButtonGroup();
		JMenuItem sparqlMenuItem = new JRadioButtonMenuItem("SPARQL");
		sparqlMenuItem.setSelected(true);
		sparqlMenuItem.addActionListener( new SelectSyntaxAction(Syntax.syntaxSPARQL) );
		group.add(sparqlMenuItem);
		syntaxMenuItem.add(sparqlMenuItem);

		JMenuItem arqMenuItem = new JRadioButtonMenuItem("ARQ");
		arqMenuItem.addActionListener( new SelectSyntaxAction(Syntax.syntaxARQ) );
		group.add(arqMenuItem);
		syntaxMenuItem.add(arqMenuItem);

		this.add(syntaxMenuItem);
		
		this.addSeparator();
		
		JMenuItem helpMenuItem = 
				createMenuItem("SPARQL-Sheet",0,null,null,new HelpAction("SparqlCheatSheet.txt"));
		this.add(helpMenuItem);
		JMenuItem aideMenuItem = 
				createMenuItem("Aide-SPARQL",0,null,null,new HelpAction("SparqlRappels.txt"));
		this.add(aideMenuItem);
		
	}

	static class SelectSyntaxAction extends AbstractAction {
		private Syntax syntax;
		
		public SelectSyntaxAction(Syntax syntax) {
			this.syntax = syntax;
		}
		
		public void actionPerformed(ActionEvent e) {
			ComponentRegistry.registry.setSyntax(syntax);
		}
	}
	
	static class HelpAction extends AbstractAction{
		private String fileName;
		HelpAction(String fileName){
			this.fileName=fileName;
		}
		
		@Override
		public void actionPerformed(ActionEvent e) {
			try {
                FileInputStream fin = new FileInputStream(fileName);
                String help = FileManager.get().readWholeFileAsUTF8(fin) ;
				JEditorPane editPane = new JEditorPane("text/plain",help);
				editPane.setEditable(false);
				editPane.setFont(new Font(Font.MONOSPACED,Font.PLAIN,12));
				editPane.setCaretPosition(0);
				JInternalFrame pane=new JInternalFrame("SPARQL 1.1 - Notes",true,true,true,true);
		        pane.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
				pane.add(new JScrollPane(editPane));
				pane.setSize(600,400);
				pane.setLocation(400, 50);
				pane.setVisible(true);
			    ComponentRegistry.registry.getMainWindow().getDesktop().add(pane);
			} catch (FileNotFoundException ex) {
				ex.printStackTrace();
			}
		}
		
	}
}
